from bqmigratrr.commands.BigQuery import BigQuery
from google.api_core.exceptions import Conflict

import os
import logging


bqMigrator = BigQuery("massage-stg-85cc4")

def up(migration, name: str):
    try:
        logging.info("Attempting to run migration for {}".format(name))
        migration.up(bqMigrator)
        bqMigrator.insertMigrationRow(name)
    except Conflict:
        logging.warning(
            "There was a conflict, {} may have already been run. Skipping".format(name))



from bqmigratrr.enums.DataType import DataType
from bqmigratrr.enums.Mode import Mode


class Column:
    def __init__(self, name: str, datatype: DataType, mode: Mode):
        self.datatype = datatype
        self.mode = mode
        self.name = name

        # if datatype == DataType.record:
        #     if len(columns) == 0:
        #         raise Exception("Columns must be defined when using record")
        #     self.columns = columns



from bqmigratrr.enums.DataType import DataType
from bqmigratrr.enums.Mode import Mode
from bqmigratrr.models.Column import Column

from google.cloud import bigquery
import json
import os


class BigQuery(object):
    def __init__(self, project_id: str = None):
        self.migrationsDatasetName = "bqMigrations"
        self.migrationsTable = "migrations"

        if project_id is None:
            project_id = os.environ.get("gcp_project_id")

        self.client = bigquery.Client(project=project_id)
        self._createMigrationsDatasetIfNotExist()

    def createTable(self, dataset: str, tablename: str, columns: [Column]):
        if len(columns) == 0:
            raise Exception("Columns must be defined when using record")

        table_ref = self.client.dataset(dataset).table(tablename)
        bqSchema = self._generateSchema(columns)

        table = bigquery.Table(table_ref, schema=bqSchema)
        self.client.create_table(table)

    def addColumn(self, dataset: str, tablename: str, column: Column):
        self.addColumns(dataset, tablename, [column])

    def addColumns(self, dataset: str, tablename: str, columns: [Column]):
        table_ref = self.client.dataset(dataset).table(tablename)
        table = self.client.get_table(table_ref)

        original_schema = table.schema
        new_schema = original_schema[:]

        schema = self._generateSchema(columns)
        new_schema = new_schema + schema

        table.schema = new_schema
        self.client.update_table(table, ["schema"])

    def insertMigrationRow(self, migrationname: str):
        print("inserting migration row for {}".format(migrationname))
        table_ref = self.client.dataset(
            self.migrationsDatasetName).table(self.migrationsTable)
        table = self.client.get_table(table_ref)

        rows_to_insert = [
            {"id": migrationname}
        ]

        self.client.insert_rows(table, rows_to_insert)

    def _generateSchema(self, columns: [Column]) -> []:
        schema = []
        for column in columns:
            schema.append(bigquery.SchemaField(
                column.name, column.datatype.value, column.mode.value))

        return schema

    def _createMigrationsDatasetIfNotExist(self):
        self.client.create_dataset(self.migrationsDatasetName, exists_ok=True)
        table_ref = self.client.dataset(
            self.migrationsDatasetName).table(self.migrationsTable)
        bqSchema = self._generateSchema(
            [Column("id", DataType.string, Mode.required)])
        table = bigquery.Table(table_ref, schema=bqSchema)
        self.client.create_table(table, exists_ok=True)



import configparser
from bqmigratrr.commands.migrator import up
import os
from importlib.machinery import SourceFileLoader
import sys
from pathlib import Path

import coloredlogs
import logging

coloredlogs.install()

name = 'bqmigratrr'


current_dir = os.getcwd()
config = configparser.ConfigParser()

config_path = '{}/migrations.ini'.format(current_dir)

config_file = Path(config_path)
if not config_file.is_file:
    raise Exception("migrations.ini cannot be found")

config.read(config_path)
migrations_dir = config["DIRECTORIES"].get("migrations", "./migrations")


def run(migration_type: str):
    for filename in os.listdir(migrations_dir):
        # for file in files:
        if filename != "__init__.py" and filename.endswith(".py"):
            file_path = "{}/{}/{}".format(current_dir,
                                          migrations_dir, filename)
            migrationfile = SourceFileLoader(
                filename, file_path).load_module()

            if hasattr(migrationfile, "Migration"):
                logging.info("{} is a valid migration".format(filename))
                migration = migrationfile.Migration()
            
                if migration_type == "up":
                    up(migration, filename)
                # elif migration == "down":
                #     down(migration)
            else:
                logging.error("{} is not a valid migration".format(filename))


def init():
    migration_type = ""
    for index, arg in enumerate(sys.argv):
        if arg in ['--migrate']:
            migration_type = "up"
            del sys.argv[index]
            break

        if arg in ['--migrate:undo']:
            migration_type = "down"
            del sys.argv[index]
            break

        if arg in ['--migrate:test']:
            migration_type = 'test'
            del sys.argv[index]
            break

    if len(sys.argv) > 1:
        print(
            f'Usage: python {sys.argv[0]} [ --migrate|migrate:undo ]')
        raise Exception()

    run(migration_type)



import setuptools
import sys

with open("README.md", 'r') as fh:
    long_description = fh.read()

build_number = sys.argv[1]

setuptools.setup(
    name='bqmigratrr',
    version_command='git describe',
    scripts=["bin/bqmigratrr"],
    author='nick sharp',
    author_email="nick.sharp@urban.co",
    description="Big query schema migration tool",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/urbanmassage/bqmigratrr",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)



from enum import Enum


class DataType(Enum):
    string = "STRING"
    timestamp = "TIMESTAMP"
    boolean = "BOOLEAN"
    float = "FLOAT"
    integer = "INTEGER"
    numeric = "NUMERIC"
    date = "DATE"
    time = "TIME"
    datetime = "DATETIME"
    geography = "GEOGRAPHY"
    record = "RECORD"
